/*******************************************************************************


    CAMPAINHA_FALANTE.C

    Campainha para residncias com reproduo de som gravado em EEprom

    Modo PWM - utiliza o PWM interno do pic a 19,53 khz.

    Autor: Claudio Lrios

    Data: 02/01/16

    Microcontrolador: PIC 16f628a

    C/ EEprom 24c512 para guardar a mensagem:'J chegou o disco voador'

    Pwm com freq. Amostragem= 19,53khz
    No entanto, os arquivos na EEprom  foram feitos com freq. amostragem pela
    metade (9,765 khz). Isto reduz o tamanho da memria. Para obter o som
    correto cada byte de som  convertido duas vezes seguidos. Isto elimina o
    apito caracteristico na reproduo do som quando se usa arquivos com freq.
    amostragem abaixo de 15khz.

    Verso para compilador CCS C.

    Este arquivo  parte integrante do blog 'LARIOS.TECNOLOGIA.WS'
    Proposta didtica apenas, sendo testada somente em placa de protoboard.

******************************************************************************/


#include <16F628A.h>
#use delay(clock=20000000)//USE XTAL DE 20MHZ
#fuses NOWDT,HS, NOPUT, NOPROTECT, BROWNOUT, MCLR, NOLVP, NOCPD//configurao
#use fast_io(a)//modo rpido
#use fast_io(b)


#byte trisa= 0x85
#byte trisb= 0x86
#byte porta= 0x05
#byte portb= 0x06
#byte pr2=   0x92
#byte ccp1l= 0x15

#bit    scl          = 0x06.0  //rb0 pino 6
#bit    sda          = 0x06.1  //pino 7 rb1
#bit    in_opto      = 0x05.3  //pino 2 ra3

#bit    flagt2       = 0x0C.1  //flag do timer 2
#bit    som          = 0x06.3  //pino 9 saida de som
#bit    tris_som     = 0x86.3  //sentido
#bit    tris_sda     = 0x86.1  //rb1 sentido

int8 const  tcl      =  1;


#define  n_rep  2  //repete 2 vezes a frase 


/*==============================================================================
       SUB-ROTINAS   I2C
==============================================================================*/

/*==============================================================================
                            START
==============================================================================*/
void start_i2c(){
  tris_sda=1; //alta impedancia
  scl=1; //
  delay_us(tcl);
  sda=0;
  tris_sda=0; //saida zerada
  delay_us(tcl);
  scl=0;  //zera scl
  delay_us(tcl);
  tris_sda=1; //alta impedancia
}
/*==============================================================================
                            STOP
==============================================================================*/


void stop_i2c(){
  sda=0;
  tris_sda=0;
  scl=0;
  delay_us(tcl);
  scl=1;
  delay_us(tcl);
  tris_sda=1;//sda =1 (por alta impedancia + resistor pullup)
}
/*==============================================================================
                            ACKNOLEDGE
==============================================================================*/

void ack(){
   sda=0;
   tris_sda=0;
   scl=1;
   delay_us(tcl);
   scl=0;
   delay_us(tcl);
   tris_sda=1;//sda =1 (por alta impedancia + resistor pullup)

}
/*==============================================================================
                          NO_ACKNOLEDGE
==============================================================================*/
void no_ack(){

   tris_sda=1; //sda =1 (por alta impedancia + resistor pullup)
   scl=1;
   delay_us(tcl);
   scl=0;
   delay_us(tcl);
  }
/*==============================================================================
                      ENVIA BYTE PARA EEPROM 24C256/512
==============================================================================*/
void send_i2c(int b){
int a;
  tris_sda=0;
  for(a=0;a<8;a++){
  if(b&0x80){ sda=1;} else {sda=0;}
  scl=1;
  delay_us(tcl);
  scl=0;
  b=b<<1;
  }

}

/*==============================================================================
     ROTINA DE REPRODUO DE SOM GRAVADO NA MEMRIA EEPROM EXTERNA
==============================================================================*/

void reproduzir(int16 init,int16 end){
 int8 buf,d;




   start_i2c(); //inicia condio de start
   send_i2c(0xa0);//device write
   ack();
   send_i2c(make8(init,1));// endereo alto a ser reproduzido
   ack();
   send_i2c(make8(init,0));//endereo baixo a ser reproduzido
   ack();

   start_i2c(); //inicia condio de start
   send_i2c(0xa1);//device read
   ack();
   tris_sda=1; //entrada

  tris_som=0;//port de som como sada
  for(;;){// reproduz at encontrar endereo final
       for(d=0;d<8;d++){
         shift_left(&buf,1,sda); //recebe o mais significativo primeiro
         scl=1;
         delay_us(tcl);//fornece um clock
         scl=0;
      }
     
      if(++init==end) {tris_som=1; no_ack(); stop_i2c(); break;} //para no fim do arquivo
       ack();//responde

      while(!flagt2) ;//aguarda 'pwm' finalizar processo
      ccp1l=buf;//carga do valor do pwm
      flagt2=0;//zera flag tmr2


      while(!flagt2) ;//aguarda 'pwm' finalizar processo
      ccp1l=buf;//repete carga do valor do pwm para usar arquivo de 9765hz
      flagt2=0;//zera flag tmr2

  }
      tris_som=1;
      
}

/*==============================================================================

                      ROTINA MAIN

==============================================================================*/

void main() {

   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);//timer 0 no usado
   setup_timer_1(T1_DISABLED);//desliga timer 1
   setup_timer_2 ( T2_DIV_BY_1, 0, 1);//seta timer 2 para gerar freq.pwm
   setup_comparator(NC_NC_NC_NC);//desliga comparadores internos
   setup_ccp1(CCP_PWM);//ativa modulo ccp para pwm
   setup_vref(FALSE);//desliga voltagem de referncia
   trisa=0b11111111;//ajusta entradas e sadas dos ports
   trisb=0b11110100;//idem
   pr2=0xff;//periodo max
   delay_ms(1000);//delay inicial 

/*==============================================================================

                      LOOP PRINCIPAL

==============================================================================*/
  for(;;){
/*******************************************************************************
REPRODUZ ARQUIVO GRAVADO PREVIAMENTE NA EEPROM 24C512
*******************************************************************************/
   unsigned int8 a;
      
      if(!in_opto) {// se campainha foi acionada
      
       for(a=0;a<n_rep;a++){
       reproduzir(0,0x5cb3);//faixa musical inicial
       
       reproduzir(0x5cb4,0xace7);//voz: "J chegou o disco voador!"
           
       }
       reproduzir(0,0x5cb3);//faixa musica final
      
       while(!in_opto) delay_ms(500);//interruptor da campainha tem que ser solto
      }


   }//for(;;)


}//main
