/*******************************************************************************


    CAMPAINHA_FALANTE.C

    Campainha para residncias com reproduo de som gravado em EEprom

    Modo PWM - utiliza o PWM interno do pic a 19,53 khz.

    Autor: Claudio Lrios

    Data: 02/01/16

    Microcontrolador: PIC 16f628a

    C/ EEprom 24c512 para guardar a mensagem:'J chegou o disco voador'

    Pwm com freq. Amostragem= 19,53khz
    No entanto, os arquivos na EEprom  foram feitos com freq. amostragem pela
    metade (9,765 khz). Isto reduz o tamanho da memria. Para obter o som
    correto cada byte de som  convertido duas vezes seguidos. Isto elimina o
    apito caracteristico na reproduo do som quando se usa arquivos com freq.
    amostragem abaixo de 15khz.
    
    Verso XC8 Compiler (MplabX)

    Este arquivo  parte integrante do blog 'LARIOS.TECNOLOGIA.WS'
    Proposta didtica apenas, sendo testada somente em placa de protoboard.

******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#define  _XTAL_FREQ  20000000
#include <xc.h>

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

// CONFIG
#pragma config FOSC = HS        // Oscillator Selection bits (HS oscillator: High-speed crystal/resonator on RA6/OSC2/CLKOUT and RA7/OSC1/CLKIN)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable bit (PWRT disabled)
#pragma config MCLRE = ON       // RA5/MCLR/VPP Pin Function Select bit (RA5/MCLR/VPP pin function is MCLR)
#pragma config BOREN = ON       // Brown-out Detect Enable bit (BOD enabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable bit (RB4/PGM pin has digital I/O function, HV on MCLR must be used for programming)
#pragma config CPD = OFF        // Data EE Memory Code Protection bit (Data memory code protection off)
#pragma config CP = OFF         // Flash Program Memory Code Protection bit (Code protection off)



#define    scl         RB0  //rb0 pino 6
#define    sda         RB1  //pino 7 rb1
#define    in_opto     RA3  //pino 2 ra3

#define    flagt2       TMR2IF  //flag do timer 2
#define    som          RB3  //pino 9 saida de som
#define    tris_som     TRISB3  //sentido
#define    tris_sda     TRISB1  //rb1 sentido

#define   tcl    1


#define  n_rep  2  //repete 2 vezes a frase 


/*==============================================================================
       SUB-ROTINAS   I2C
==============================================================================*/

/*==============================================================================
                            START
==============================================================================*/
void start_i2c(){
  tris_sda=1; //alta impedancia
  scl=1; //
  __delay_us(tcl);
  sda=0;
  tris_sda=0; //saida zerada
  __delay_us(tcl);
  scl=0;  //zera scl
  __delay_us(tcl);
  tris_sda=1; //alta impedancia
}
/*==============================================================================
                            STOP
==============================================================================*/


void stop_i2c(){
  sda=0;
  tris_sda=0;
  scl=0;
  __delay_us(tcl);
  scl=1;
  __delay_us(tcl);
  tris_sda=1;//sda =1 (por alta impedancia + resistor pullup)
}
/*==============================================================================
                            ACKNOLEDGE
==============================================================================*/

void ack(){
   sda=0;
   tris_sda=0;
   scl=1;
   __delay_us(tcl);
   scl=0;
   __delay_us(tcl);
   tris_sda=1;//sda =1 (por alta impedancia + resistor pullup)

}
/*==============================================================================
                          NO_ACKNOLEDGE
==============================================================================*/
void no_ack(){

   tris_sda=1; //sda =1 (por alta impedancia + resistor pullup)
   scl=1;
   __delay_us(tcl);
   scl=0;
   __delay_us(tcl);
  }
/*==============================================================================
                      ENVIA BYTE PARA EEPROM 24C256/512
==============================================================================*/
void send_i2c(int b){
char a;
  tris_sda=0;
  for(a=0;a<8;a++){
  if(b&0x80){ sda=1;} else {sda=0;}
  scl=1;
  __delay_us(tcl);
  scl=0;
  b=b<<1;
  }

}

/*==============================================================================
     ROTINA DE REPRODUO DE SOM GRAVADO NA MEMRIA EEPROM EXTERNA
==============================================================================*/

void reproduzir(int init,int end){
 char buf,d;




   start_i2c(); //inicia condio de start
   send_i2c(0xa0);//device write
   ack();
   send_i2c(init>>8);// endereo alto a ser reproduzido
   ack();
   send_i2c(init);//endereo baixo a ser reproduzido
   ack();

   start_i2c(); //inicia condio de start
   send_i2c(0xa1);//device read
   ack();
   tris_sda=1; //entrada

  tris_som=0;//port de som como sada
  for(;;){// reproduz at encontrar endereo final
       for(d=0;d<8;d++){
         buf=buf<<1;//desloca para esquerda
         if(sda) buf++;
         scl=1;
         __delay_us(tcl);//fornece um clock
         scl=0;
      }
     
      if(++init==end) {tris_som=1; no_ack(); stop_i2c(); break;} //para no fim do arquivo
       ack();//responde

      while(!flagt2) ;//aguarda 'pwm' finalizar processo
      CCPR1L=buf;//carga do valor do pwm
      flagt2=0;//zera flag tmr2


      while(!flagt2) ;//aguarda 'pwm' finalizar processo
      CCPR1L=buf;//repete carga do valor do pwm para usar arquivo de 9765hz
      flagt2=0;//zera flag tmr2

  }
      tris_som=1;
      
}

/*==============================================================================

                      ROTINA MAIN

==============================================================================*/

int main(int argc, char** argv) {

   CMCON=7;//desliga comparadores
   T2CON=0B00000100;//POSTCALE=/1;LIGA TIMER2; PRESCALLER=/1  
   CCP1CON=0B00001100; //Modo PWM
   TRISA=0;
   TRISB=0;
   TRISA=0b11111111;
   TRISB=0b11110100;
   PR2=0xff;//periodo max
   __delay_ms(100);


/*==============================================================================

                      LOOP PRINCIPAL

==============================================================================*/
  for(;;){
/*******************************************************************************
REPRODUZ ARQUIVO GRAVADO PREVIAMENTE NA EEPROM 24C512
*******************************************************************************/
    char a;
      
      if(!in_opto) {// se campainha foi acionada
      
       for(a=0;a<n_rep;a++){
       reproduzir(0,0x5cb3);//faixa musical inicial
       
       reproduzir(0x5cb4,0xace7);//voz: "J chegou o disco voador!"
           
       }
       reproduzir(0,0x5cb3);//faixa musica final
      
       while(!in_opto) __delay_ms(500);//interruptor da campainha tem que ser solto
      }


   }//for(;;)
return (EXIT_SUCCESS);

}//main
